#include "StdAfx.h"
#include "ScriptletContainerFactory.h"
#include "ListBox.h"
#include "ComponentPropertyAccessor.h"
#include "Binding.h"
#include "DataContext.h"
#include "ComponentScriptletPlugin.h"

IMPLEMENT_SINGLETON_CODE(CScriptletContainerFactory);
CScriptletContainerFactory::CScriptletContainerFactory(void)
{
}

CScriptletContainerFactory::~CScriptletContainerFactory(void)
{
}
ILayoutManager* CScriptletContainerFactory::CreateLayout(LPCTSTR pszcID)
{
	ILayoutManager* pLayout = reinterpret_cast<ILayoutManager*>(m_ScriptContainerFactory.CreateObject(pszcID));
	return pLayout;
}
IActionListener* CScriptletContainerFactory::CreateAction(LPCTSTR pszcID)
{
	IActionListener* pAction = reinterpret_cast<IActionListener*>(m_ScriptContainerFactory.CreateObject(pszcID));
	return pAction;
}
IWindowListener* CScriptletContainerFactory::CreateWindowAction(LPCTSTR pszcID)
{
	IWindowListener* pAction = reinterpret_cast<IWindowListener*>(m_ScriptContainerFactory.CreateObject(pszcID));
	return pAction;
}
IPropertyChangedListener* CScriptletContainerFactory::CreatePropertyChangeListener(LPCTSTR pszcID)
{
	IPropertyChangedListener* pPropertyListener = reinterpret_cast<IPropertyChangedListener*>(m_ScriptContainerFactory.CreateObject(pszcID));
	return pPropertyListener;
}
IItemListener* CScriptletContainerFactory::CreateitemListener(LPCTSTR pszcID)
{
	IItemListener* pItemListener = reinterpret_cast<IItemListener*>(m_ScriptContainerFactory.CreateObject(pszcID));
	return pItemListener;
}

ComponentPropertyAccessor* CScriptletContainerFactory::CreatePropertyAccessor(LPCTSTR pszcID)
{
	ComponentPropertyAccessor* pPropertyAccessor = reinterpret_cast<ComponentPropertyAccessor*>(m_ComponentPropertyAccessorFactory.CreateObject(pszcID));
	return pPropertyAccessor;
}

CContainer* CScriptletContainerFactory::CreateScriptContainer(LPCTSTR pszcID)
{
	CContainer* pScriptContainer = reinterpret_cast<CContainer*>(m_ComponentCreatorFactory.CreateObject(pszcID));
	return pScriptContainer;
}
CScriptletDialog* CScriptletContainerFactory::CreateScriptDialog(LPCTSTR pszcID)
{
	CScriptletDialog* pScriptDialog = reinterpret_cast<CScriptletDialog*>(m_ComponentCreatorFactory.CreateObject(pszcID));
	return pScriptDialog;
}
CBaseWindow* CScriptletContainerFactory::CreateBaseWindow(LPCTSTR pszcID)
{
	CBaseWindow* pBaseWindow = reinterpret_cast<CBaseWindow*>(m_ComponentCreatorFactory.CreateObject(pszcID));
	return pBaseWindow;
}

CComponent* CScriptletContainerFactory::MakeComponent(CContainer* pContainer, TiXmlElement* pNode)
{
	CComponent* pComponent = NULL;
	ATLASSERT(pNode);

	USES_CONVERSION;
	LPCTSTR pszcElementName = A2CT(pNode->Value());
	if (_tcslen(pszcElementName) > 0)
	{
		CComponentScriptletPlugin* pPlugin = reinterpret_cast<CComponentScriptletPlugin*>(m_ScriptPluginFactory.CreateObject(pszcElementName));
		if (pPlugin)
		{
			pComponent = pPlugin->CreateComponent(pContainer, pNode);
			pPlugin->AutoRelease();
		}		
	}	
	return pComponent;
}
BOOL CScriptletContainerFactory::GenerateComponent(CContainer* pContainer, CComponent* pComponent, TiXmlElement* pNode)
{
	ATLASSERT(pNode);

	BOOL bSuccess = FALSE;
	USES_CONVERSION;
	LPCTSTR pszcElementName = A2CT(pNode->Value());
	if (_tcslen(pszcElementName) > 0)
	{
		CComponentScriptletPlugin* pPlugin = reinterpret_cast<CComponentScriptletPlugin*>(m_ScriptPluginFactory.CreateObject(pszcElementName));
		if (pPlugin)
		{			
			if (pPlugin->GenerateComponent(pContainer, pComponent, pNode))
			{
				pComponent->RecalcLayout();
				bSuccess = TRUE;
			}
			pPlugin->AutoRelease();
		}
	}
	return bSuccess;
}
CComponent* CScriptletContainerFactory::CreateComponent(CContainer* pContainer, TiXmlElement* pNode)
{
	CComponent* pComponent = MakeComponent(pContainer, pNode);
	if (pComponent != NULL)
	{
		GenerateComponent(pContainer, pComponent, pNode);
	}
	return pComponent;
}
IDataContext* CScriptletContainerFactory::FindDataContext(LPCTSTR pszcID)
{
	CAtlMap<CString, IDataContext*>::CPair* pair = m_DataContexts.Lookup(CString(pszcID));
	if (pair != NULL)
	{
		return pair->m_value;
	}
	return NULL;
}
IValueConverter* CScriptletContainerFactory::CreateValueConverter(LPCTSTR pszcID)
{
	IValueConverter* pValueConverter = reinterpret_cast<IValueConverter*>(m_ValueConverterFactory.CreateObject(pszcID));
	return pValueConverter;
}